package de.geeksfactory.opacclient.frontend;

import org.acra.ACRA;

import android.content.Intent;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.Window;
import de.geeksfactory.opacclient.NotReachableException;
import de.geeksfactory.opacclient.OpacClient;
import de.geeksfactory.opacclient.OpacTask;
import de.geeksfactory.opacclient.R;
import de.geeksfactory.opacclient.apis.OpacApi;
import de.geeksfactory.opacclient.apis.OpacApi.OpacErrorException;
import de.geeksfactory.opacclient.objects.SearchRequestResult;

/**
 * An activity representing a list of SearchResults. This activity has different
 * presentations for handset and tablet-size devices. On handsets, the activity
 * presents a list of items, which when touched, lead to a
 * {@link SearchResultDetailActivity} representing item details. On tablets, the
 * activity presents the list of items and item details side-by-side using two
 * vertical panes.
 * <p>
 * The activity makes heavy use of fragments. The list of items is a
 * {@link SearchResultListFragment} and the item details (if present) is a
 * {@link SearchResultDetailFragment}.
 * <p>
 * This activity also implements the required
 * {@link SearchResultListFragment.Callbacks} interface to listen for item
 * selections.
 */
public class SearchResultListActivity extends OpacActivity implements
		SearchResultListFragment.Callbacks,
		SearchResultDetailFragment.Callbacks {

	/**
	 * Whether or not the activity is in two-pane mode, i.e. running on a tablet
	 * device.
	 */
	protected boolean mTwoPane;

	protected SearchRequestResult searchresult;

	protected SearchStartTask st;

	protected SearchResultListFragment listFragment;
	protected SearchResultDetailFragment detailFragment;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		supportRequestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
		super.onCreate(savedInstanceState);
		// Show the Up button in the action bar.
		getSupportActionBar().setDisplayHomeAsUpEnabled(true);

		listFragment = (SearchResultListFragment) getSupportFragmentManager()
				.findFragmentById(R.id.searchresult_list);

		if (findViewById(R.id.searchresult_detail_container) != null) {
			// The detail container view will be present only in the
			// large-screen layouts (res/values-large and
			// res/values-sw600dp). If this view is present, then the
			// activity should be in two-pane mode.
			mTwoPane = true;

			// In two-pane mode, list items should be given the
			// 'activated' state when touched.
			listFragment.setActivateOnItemClick(true);
		}

		if (savedInstanceState == null) {
			performsearch();
		}
	}

	public void performsearch() {
		st = new SearchStartTask();
		st.execute(app, getIntent().getBundleExtra("query"));
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		if (item.getItemId() == android.R.id.home) {
			finish();
			return true;
		}
		return super.onOptionsItemSelected(item);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater mi = new MenuInflater(this);
		mi.inflate(R.menu.activity_search_results, menu);

		return super.onCreateOptionsMenu(menu);
	}

	/**
	 * Callback method from {@link SearchResultListFragment.Callbacks}
	 * indicating that the item with the given ID was selected.
	 */
	@Override
	public void onItemSelected(int nr, String id, int pageToLoad) {
		if((app.getApi().getSupportFlags() & OpacApi.SUPPORT_FLAG_ENDLESS_SCROLLING) == 0 &&
				pageToLoad != listFragment.getLastLoadedPage()) {
			new ReloadOldPageTask().execute(app, pageToLoad, nr, id);
		} else {
			showDetail(nr, id);
		}
	}
	
	public void showDetail(int nr, String id) {		
		if (mTwoPane) {
			// In two-pane mode, show the detail view in this activity by
			// adding or replacing the detail fragment using a
			// fragment transaction.
			Bundle arguments = new Bundle();
			arguments.putInt(SearchResultDetailFragment.ARG_ITEM_NR, nr);
			if (id != null)
				arguments.putString(SearchResultDetailFragment.ARG_ITEM_ID, id);
			detailFragment = new SearchResultDetailFragment();
			detailFragment.setArguments(arguments);
			getSupportFragmentManager()
					.beginTransaction()
					.replace(R.id.searchresult_detail_container, detailFragment)
					.commit();

		} else {
			// In single-pane mode, simply start the detail activity
			// for the selected item ID.
			Intent detailIntent = new Intent(this,
					SearchResultDetailActivity.class);
			detailIntent.putExtra(SearchResultDetailFragment.ARG_ITEM_NR, nr);
			if (id != null)
				detailIntent.putExtra(SearchResultDetailFragment.ARG_ITEM_ID,
						id);
			startActivity(detailIntent);
		}
	}

	public class SearchStartTask extends OpacTask<SearchRequestResult> {
		protected Exception exception;

		@Override
		protected SearchRequestResult doInBackground(Object... arg0) {
			super.doInBackground(arg0);
			Bundle query = (Bundle) arg0[1];

			try {
				SearchRequestResult res = app.getApi().search(query);
				// Load cover images, if search worked and covers available
				return res;
			} catch (java.net.UnknownHostException e) {
				exception = e;
				e.printStackTrace();
			} catch (java.net.SocketException e) {
				exception = e;
			} catch (OpacErrorException e) {
				exception = e;
			} catch (Exception e) {
				exception = e;
				ACRA.getErrorReporter().handleException(e);
			}

			return null;
		}

		@Override
		protected void onPostExecute(SearchRequestResult result) {
			if (result == null) {

				if (exception instanceof OpacErrorException) {
					if (exception.getMessage().equals("is_a_redirect")) {
						// Some libraries (SISIS) do not show a result list if
						// only one result
						// is found but instead directly show the result
						// details.
						Intent intent = new Intent(
								SearchResultListActivity.this,
								SearchResultDetailActivity.class);
						intent.putExtra(SearchResultDetailFragment.ARG_ITEM_ID,
								(String) null);
						startActivity(intent);
						finish();
						return;
					}

					listFragment.showConnectivityError(exception.getMessage());
				} else if (exception instanceof NotReachableException)
					listFragment.showConnectivityError(getResources()
							.getString(R.string.connection_error_detail_nre));
				else
					listFragment.showConnectivityError();
			} else {
				searchresult = result;
				loaded();
			}
		}
	}
	
	public class ReloadOldPageTask extends OpacTask<SearchRequestResult> {
		int nr;
		String id;
		Integer page;
		Exception exception;
		
		@Override
		protected void onPreExecute() {
			setProgressBarIndeterminateVisibility(true);
		}
		
		@Override
		protected SearchRequestResult doInBackground(Object... arg0) {
			page = (Integer) arg0[1];
			nr = (Integer) arg0[2];
			id = (String) arg0[3];
			OpacClient app = (OpacClient) arg0[0];

			try {
				SearchRequestResult res = app.getApi().searchGetPage(page);
				return res;
			} catch (java.net.UnknownHostException e) {
				exception = e;
				e.printStackTrace();
			} catch (java.net.SocketException e) {
				exception = e;
			} catch (OpacErrorException e) {
				exception = e;
			} catch (Exception e) {
				exception = e;
				ACRA.getErrorReporter().handleException(e);
			}

			return null;
		}
		
		@Override
		protected void onPostExecute(SearchRequestResult result) {
			setProgressBarIndeterminateVisibility(false);
			if (result == null) {

				if (exception instanceof OpacErrorException) {
					listFragment.showConnectivityError(exception.getMessage());
				} else if (exception instanceof NotReachableException)
					listFragment.showConnectivityError(getResources()
							.getString(R.string.connection_error_detail_nre));
				else
					listFragment.showConnectivityError();
			} else {
				//Everything ran correctly, show Detail
				listFragment.setLastLoadedPage(page);
				showDetail(nr, id);
			}
		}
	}

	protected void loaded() {
		try {
			listFragment.setListShown(true);
			listFragment.setSearchResult(searchresult);
		} catch (IllegalStateException e) {
			e.printStackTrace();
		}
	}

	@Override
	protected int getContentView() {
		return R.layout.activity_searchresult_list;
	}

	@Override
	public void removeFragment() {
		getSupportFragmentManager().beginTransaction().remove(detailFragment)
				.commit();
	}

	@Override
	public void reload() {
		performsearch();
	}
}
